@@ -42,7 +42,7 @@ from ajsonapi.requests import (
     parse_relationship_document_query,
     parse_relationship_query,
 )
-from ajsonapi.responses import errors_response
+from ajsonapi.responses import errors_response, no_content_response
 from ajsonapi.table import create_all as table_create_all
 from ajsonapi.table import parse_model
 from ajsonapi.uri.relationship import parse as parse_relationship
@@ -54,6 +54,8 @@ class Application:
     Default routes are derived from the specified object model.
     """
 
+    # pylint: disable=too-many-public-methods
+
     def __init__(self, *args, **kwargs):
         Event.connect(self)
         self.app = web.Application(*args, **kwargs)
@@ -103,6 +105,7 @@ class Application:
         self.app.add_routes([
             web.get(path, self.get_collection),
             web.post(path, self.post_collection),
+            web.options(path, self.options_collection),
         ])
 
         path = r'/{collection:[^/]*}/{id:[0-9A-Fa-f-]*}'
@@ -110,6 +113,7 @@ class Application:
             web.get(path, self.get_object),
             web.delete(path, self.delete_object),
             web.patch(path, self.patch_object),
+            web.options(path, self.options_object),
         ])
 
         path = (r'/{collection:[^/]*}/{id:[0-9A-Fa-f-]*}/'
@@ -120,12 +124,14 @@ class Application:
             web.post(path, self.post_relationship),
             web.delete(path, self.delete_relationship),
             web.put(path, self.put_relationship),
+            web.options(path, self.options_relationship),
         ])
 
         path = (r'/{collection:[^/]*}/{id:[0-9A-Fa-f-]*}/'
                 r'{related_resource:[^/]*}')
         self.app.add_routes([
             web.get(path, self.get_related_resource),
+            web.options(path, self.options_related_resource),
         ])
 
     def add_event_socket_route(self):
@@ -168,6 +174,15 @@ class Application:
         except ErrorsException as err:
             return errors_response(err.errors)
 
+    async def options_collection(self, request):
+        """Entry point for OPTIONS /{collection} requests.
+
+        Args:
+            request: The OPTIONS /{collection} request.
+        """
+        # pylint: disable=unused-argument
+        return no_content_response(headers={'Allow': 'GET,OPTIONS,POST'})
+
     async def get_object(self, request):
         """Entry point for GET /{collection}/{id} requests.
 
@@ -207,6 +222,16 @@ class Application:
         except ErrorsException as err:
             return errors_response(err.errors)
 
+    async def options_object(self, request):
+        """Entry point for OPTIONS /{collection}/{id} requests.
+
+        Args:
+            request: The OPTIONS /{collection}/{id} request.
+        """
+        # pylint: disable=unused-argument
+        return no_content_response(
+            headers={'Allow': 'DELETE,GET,OPTIONS,PATCH'})
+
     async def get_relationship(self, request):
         """Entry point for GET /{collection}/{id}/relationships/{relationship}
         requests.
@@ -293,6 +318,20 @@ class Application:
         relationship = parse_relationship(request)
         return relationship.put()
 
+    async def options_relationship(self, request):
+        """Entry point for OPTIONS
+        /{collection}/{id}/relationships/{relationship} requests.
+
+        Args:
+            request: The OPTIONS /{collection}/{id}/relationships/{relationship}
+                request.
+
+        Returns:
+            Appropriate response for the request.
+        """
+        relationship = parse_relationship(request)
+        return relationship.options()
+
     async def get_related_resource(self, request):
         """Entry point for GET /{collection}/{id}/{related_resource} requests.
 
@@ -314,6 +353,19 @@ class Application:
         except ErrorsException as err:
             return errors_response(err.errors)
 
+    async def options_related_resource(self, request):
+        """Entry point for OPTIONS /{collection}/{id}/{related_resource}
+        requests.
+
+        Args:
+            request: The OPTIONS /{collection}/{id}/{related_resource} request.
+
+        Returns:
+            Appropriate response for the request.
+        """
+        # pylint: disable=unused-argument
+        return no_content_response(headers={'Allow': 'GET,OPTIONS'})
+
     async def get_event_socket(self, request):
         """Creates a web socket for events."""
 
