@@ -23,6 +23,7 @@
 from abc import abstractmethod
 from asyncio import Event, Lock, create_task, gather
 from itertools import chain
+from json import loads
 from re import compile as re_compile
 
 from asyncpg.exceptions import (
@@ -56,6 +57,7 @@ from ajsonapi.postgres import (
 )
 from ajsonapi.query import parse_fields, parse_page, query_has_parameter
 from ajsonapi.responses import document_response, no_content_response
+from ajsonapi.types import Json
 from ajsonapi.uri.resource import Resource
 
 RE_FILTER = re_compile(r'filter\[([^\]]*)\]')
@@ -241,6 +243,10 @@ class Collection(Resource):
         await env.run()
         id_name = id_number_to_name(env.id_number)
         document.data['id'] = id_name
+        for attr_name, value in document.attributes.items():
+            attr = self.table.attributes_by_name[attr_name]
+            if attr.type_ == Json:
+                document.attributes[attr_name] = loads(value)
         path = f'{self.path()}/{id_name}'
         document.data['links'] = {'self': path}
         event = CreateEvent(document.data)
@@ -264,7 +270,7 @@ class Collection(Resource):
         length = 1 + len(env.attributes) + len(lfkeys)
         column_values = ', '.join(
             [f'${index}' for index in range(1, length + 1)])
-        stmt = (f'INSERT INTO {self.table.name} ({column_names}) '
+        stmt = (f'INSERT INTO data.{self.table.name} ({column_names}) '
                 f'VALUES ({column_values})')
         try:
             async with env.lock:
@@ -302,10 +308,10 @@ class Collection(Resource):
         column_values = ', '.join(
             [f'${index}' for index in range(1, length + 1)])
         if column_values:
-            stmt = (f'INSERT INTO {self.table.name} ({column_names}) '
+            stmt = (f'INSERT INTO data.{self.table.name} ({column_names}) '
                     f'VALUES ({column_values}) RETURNING id')
         else:
-            stmt = (f'INSERT INTO {self.table.name} '
+            stmt = (f'INSERT INTO data.{self.table.name} '
                     'DEFAULT VALUES RETURNING id')
         try:
             async with env.lock:
@@ -406,18 +412,22 @@ class Collection(Resource):
         sort = env.query['sort']
         if filters:
             if sort:
-                stmt = (f"SELECT {', '.join(columns)} FROM {self.table.name} "
-                        f"WHERE {' AND '.join(filters)} "
-                        f"ORDER BY {', '.join(sort)}")
+                stmt = (
+                    f"SELECT {', '.join(columns)} FROM data.{self.table.name} "
+                    f"WHERE {' AND '.join(filters)} "
+                    f"ORDER BY {', '.join(sort)}")
             else:
-                stmt = (f"SELECT {', '.join(columns)} FROM {self.table.name} "
-                        f"WHERE {' AND '.join(filters)}")
+                stmt = (
+                    f"SELECT {', '.join(columns)} FROM data.{self.table.name} "
+                    f"WHERE {' AND '.join(filters)}")
         else:
             if sort:
-                stmt = (f"SELECT {', '.join(columns)} FROM {self.table.name} "
-                        f"ORDER BY {', '.join(sort)}")
+                stmt = (
+                    f"SELECT {', '.join(columns)} FROM data.{self.table.name} "
+                    f"ORDER BY {', '.join(sort)}")
             else:
-                stmt = f"SELECT {', '.join(columns)} FROM {self.table.name}"
+                stmt = (
+                    f"SELECT {', '.join(columns)} FROM data.{self.table.name}")
         page = env.query['page']
         if page:
             limit = page['size']
@@ -443,7 +453,7 @@ class Collection(Resource):
         else:
             columns = (str(col) for col in self.table.columns)
         stmt = (f"SELECT {', '.join(columns)} "
-                f"FROM {self.table.name} WHERE id IN ({ids})")
+                f"FROM data.{self.table.name} WHERE id IN ({ids})")
         ### Optimize the stmt!
         async with env.lock:
             log.debug(stmt)
@@ -465,7 +475,7 @@ class Collection(Resource):
         else:
             columns = (str(col) for col in self.table.columns)
         stmt = (f"SELECT {', '.join(columns)} "
-                f"FROM {self.table.name} WHERE id IN ({ids})")
+                f"FROM data.{self.table.name} WHERE id IN ({ids})")
         ### Optimize the stmt!
         async with env.lock:
             log.debug(f'{stmt}: {env.object_id}')
@@ -479,13 +489,17 @@ class Collection(Resource):
         """Converts a asyncpg record into a JSON API data member."""
         attributes = {}
         relationships = {}
-        for col, value in record.items():
-            if col == 'id':
+        for col_name, value in record.items():
+            if col_name == 'id':
                 id_name = id_number_to_name(value)
-            elif col in self.table.attribute_names:
-                attributes[col] = value
-            else:  # col is a lfkey
-                rel = self.table.relationship_by_lfkey[col]
+            elif col_name in self.table.attributes_by_name:
+                col = self.table.attributes_by_name[col_name]
+                if col.type_ == Json:  # Should we use a virtual function?
+                    attributes[col_name] = loads(value)
+                else:
+                    attributes[col_name] = value
+            else:  # col_name is a lfkey
+                rel = self.table.relationship_by_lfkey[col_name]
                 if value:
                     relationship_data = {
                         'type': rel.rtable.collection.name,
