@@ -23,6 +23,7 @@
 from abc import abstractmethod
 from asyncio import Event, Lock, create_task, gather
 from itertools import chain
+from json import loads
 
 from ajsonapi.conversions import id_number_to_name
 from ajsonapi.environments import GetObjectIncludedEnvironment
@@ -34,6 +35,7 @@ from ajsonapi.exceptions import ErrorsException
 from ajsonapi.id_value import IdValue
 from ajsonapi.logging import log
 from ajsonapi.responses import document_response
+from ajsonapi.types import Json
 from ajsonapi.uri.collection import Collection
 from ajsonapi.uri.resource import Resource
 
@@ -91,7 +93,7 @@ class RelatedResource(Resource):
 
     async def verify_id_exists(self, env):
         """Raises an exception in case the id in the URL does not exist."""
-        stmt = f'SELECT FROM {self.table.name} WHERE id = $1'
+        stmt = f'SELECT FROM data.{self.table.name} WHERE id = $1'
         record = await env.connection.fetchrow(stmt, self.id.number)
         if record is None:
             env.cancel()
@@ -126,13 +128,17 @@ class ToOneRelatedResource(RelatedResource):
         """Converts a asyncpg record into a JSON API data member."""
         attributes = {}
         relationships = {}
-        for col, value in record.items():
-            if col == 'id':
+        for col_name, value in record.items():
+            if col_name == 'id':
                 id_name = id_number_to_name(value)
-            elif col in self.relationship.rtable.attribute_names:
-                attributes[col] = value
-            else:  # col is a lfkey
-                rel = self.relationship.rtable.relationship_by_lfkey[col]
+            elif col_name in self.relationship.rtable.attributes_by_name:
+                col = self.relationship.rtable.attributes_by_name[col_name]
+                if col.type_ == Json:  # Should we use a virtual function?
+                    attributes[col_name] = loads(value)
+                else:
+                    attributes[col_name] = value
+            else:  # col_name is a lfkey
+                rel = self.relationship.rtable.relationship_by_lfkey[col_name]
                 relationships[rel.name] = self.to_relationships(rel, value)
         data = {
             'type': self.relationship.rtable.collection.name,
@@ -165,10 +171,10 @@ class OneToOneLocalRelatedResource(ToOneRelatedResource):
         else:
             columns = (str(col) for col in rtable.columns)
         stmt = (f'SELECT {", ".join(columns)} '
-                f'FROM {rtable.name} '
+                f'FROM data.{rtable.name} '
                 f'WHERE id = ('
                 f' SELECT {self.relationship.lfkey}'
-                f' FROM {self.table.name}'
+                f' FROM data.{self.table.name}'
                 f' WHERE id = $1)')
         async with env.lock:
             log.debug(f'{stmt}: {self.id.number}')
@@ -199,7 +205,7 @@ class OneToOneRemoteRelatedResource(ToOneRelatedResource):
             columns = (str(col) for col in rtable.columns)
         # We could remove rfkey because we know it already.
         stmt = (f'SELECT {", ".join(columns)} '
-                f'FROM {self.relationship.rtable.name} '
+                f'FROM data.{self.relationship.rtable.name} '
                 f'WHERE {self.relationship.rfkey} = $1')
         async with env.lock:
             log.debug(f'{stmt}: {self.id.number}')
@@ -229,10 +235,10 @@ class ManyToOneRelatedResource(ToOneRelatedResource):
         else:
             columns = (str(col) for col in self.relationship.rtable.columns)
         stmt = (f'SELECT {", ".join(columns)} '
-                f'FROM {self.relationship.rtable.name} '
+                f'FROM data.{self.relationship.rtable.name} '
                 f'WHERE id = ('
                 f' SELECT {self.relationship.lfkey}'
-                f' FROM {self.table.name}'
+                f' FROM data.{self.table.name}'
                 f' WHERE id = $1)')
         async with env.lock:
             log.debug(f'{stmt}: {self.id.number}')
@@ -267,13 +273,13 @@ class ToManyRelatedResource(RelatedResource):
         """Converts a asyncpg record into a JSON API data member."""
         attributes = {}
         relationships = {}
-        for col, value in record.items():
-            if col == 'id':
+        for col_name, value in record.items():
+            if col_name == 'id':
                 id_name = id_number_to_name(value)
-            elif col in self.relationship.rtable.attribute_names:
-                attributes[col] = value
-            else:  # col is a lfkey
-                rel = self.relationship.rtable.relationship_by_lfkey[col]
+            elif col_name in self.relationship.rtable.attributes_by_name:
+                attributes[col_name] = value
+            else:  # col_name is a lfkey
+                rel = self.relationship.rtable.relationship_by_lfkey[col_name]
                 if value:
                     relationship_data = {
                         'type': rel.rtable.collection.name,
@@ -326,24 +332,24 @@ class OneToManyRelatedResource(ToManyRelatedResource):
         if filters:
             if sort:
                 stmt = (f'SELECT {", ".join(columns)} '
-                        f'FROM {self.relationship.rtable.name} '
+                        f'FROM data.{self.relationship.rtable.name} '
                         f'WHERE {self.relationship.rfkey} = $1 AND '
                         f'{" AND ".join(filters)} '
                         f'ORDER BY {", ".join(sort)}')
             else:
                 stmt = (f'SELECT {", ".join(columns)} '
-                        f'FROM {self.relationship.rtable.name} '
+                        f'FROM data.{self.relationship.rtable.name} '
                         f'WHERE {self.relationship.rfkey} = $1 AND '
                         f'{" AND ".join(filters)}')
         else:
             if sort:
                 stmt = (f'SELECT {", ".join(columns)} '
-                        f'FROM {self.relationship.rtable.name} '
+                        f'FROM data.{self.relationship.rtable.name} '
                         f'WHERE {self.relationship.rfkey} = $1 '
                         f'ORDER BY {", ".join(sort)}')
             else:
                 stmt = (f'SELECT {", ".join(columns)} '
-                        f'FROM {self.relationship.rtable.name} '
+                        f'FROM data.{self.relationship.rtable.name} '
                         f'WHERE {self.relationship.rfkey} = $1')
         async with env.lock:
             log.debug(f'{stmt}: {self.id.number}')
@@ -376,36 +382,36 @@ class ManyToManyRelatedResource(ToManyRelatedResource):
         if filters:
             if sort:
                 stmt = (f'SELECT {", ".join(columns)} '
-                        f'FROM {self.relationship.rtable.name} '
+                        f'FROM data.{self.relationship.rtable.name} '
                         f'WHERE id IN ('
                         f' SELECT {self.relationship.rafkey}'
-                        f' FROM {self.relationship.atable.name}'
+                        f' FROM data.{self.relationship.atable.name}'
                         f' WHERE {self.relationship.lafkey} = $1) AND'
                         f' {" AND ".join(filters)} '
                         f'ORDER BY {", ".join(sort)}')
             else:
                 stmt = (f'SELECT {", ".join(columns)} '
-                        f'FROM {self.relationship.rtable.name} '
+                        f'FROM data.{self.relationship.rtable.name} '
                         f'WHERE id IN ('
                         f' SELECT {self.relationship.rafkey}'
-                        f' FROM {self.relationship.atable.name}'
+                        f' FROM data.{self.relationship.atable.name}'
                         f' WHERE {self.relationship.lafkey} = $1) AND'
                         f' {" AND ".join(filters)}')
         else:
             if sort:
                 stmt = (f'SELECT {", ".join(columns)} '
-                        f'FROM {self.relationship.rtable.name} '
+                        f'FROM data.{self.relationship.rtable.name} '
                         f'WHERE id IN ('
                         f' SELECT {self.relationship.rafkey}'
-                        f' FROM {self.relationship.atable.name}'
+                        f' FROM data.{self.relationship.atable.name}'
                         f' WHERE {self.relationship.lafkey} = $1) '
                         f'ORDER BY {", ".join(sort)}')
             else:
                 stmt = (f'SELECT {", ".join(columns)} '
-                        f'FROM {self.relationship.rtable.name} '
+                        f'FROM data.{self.relationship.rtable.name} '
                         f'WHERE id IN ('
                         f' SELECT {self.relationship.rafkey}'
-                        f' FROM {self.relationship.atable.name}'
+                        f' FROM data.{self.relationship.atable.name}'
                         f' WHERE {self.relationship.lafkey} = $1)')
         async with env.lock:
             log.debug(f'{stmt}: {self.id.number}')
